#include <cmath>

#include "canvas/BackGround.hpp"
#include "canvas/BackGroundColors.hpp"
#include "Math.hpp"
#include "values/mrand.hpp"

#include <raylib.h>
#include <rlgl.h>
#include <raymath.h>

// Public
void BackGround::init(int size)
{
  this->size = size;
  sun.init();
}

void BackGround::deinit()
{
  sun.deinit();
}

void BackGround::newGen()
{
  starBuff.reset();
  mountenBuff.reset();
  m_moon = {mrand::getFloat(), mrand::getFloat(), mrand::getFloat()};
  colorSet = mrand::getValue(0, 3);
  time = floor(Remap(m_moon.y, 0, 1, 4, 0));

  BackGroundColors::setColor(colorSet, time);

  draw();
}

void BackGround::draw()
{
  if (colorSet == 3)
  {
    ClearBackground(BackGroundColors::backGroundColor);
    drawStarts();
  }
  else
  {
    DrawRectangleGradientV(0, 0, size, size, ColorAdd(BackGroundColors::backGroundColor, 60), BackGroundColors::backGroundColor);
  }

  drawSun();
  drawMounten(20, (int)(mounten1min * size), (int)(mounten1max * size), BackGroundColors::MountenColor1);
  drawMounten(21, (int)(mounten2min * size), (int)(mounten2max * size), BackGroundColors::MountenColor2);
  drawMounten(23, (int)(mounten3min * size), (int)(mounten3max * size), BackGroundColors::MountenColor3);
}

void BackGround::drawStarts()
{
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  bool star = true;

  for (size_t i = 0; i < numOfStarts; i++)
  {
    int x = starBuff.next() * size;
    int y = starBuff.next() * size;
    int weight = starBuff.next() * 3 + 1;
    float alph = Normalize(starBuff.next(), 0.04f, 0.8f);
    Color color = ColorLerp(BackGroundColors::backGroundColor, BackGroundColors::starColor, alph);

    rlColor4ub(color.r, color.g, color.b, color.a);
    if (star)
    {
      // topLeft
      rlVertex2f(x, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
    }
    else
    {
      // bottomRight
      rlVertex2f(x + weight, y + weight);
      // topRight
      rlVertex2f(x + weight, y);
      // bottomLeft
      rlVertex2f(x, y + weight);
    }
    star = !star;
  }
  rlEnd();
  rlSetTexture(0);
}

void BackGround::drawSun()
{
  int r = ((m_moon.y * (maxSizeOfMoon - minSizeOfMoon)) + minSizeOfMoon) * size;
  int xpos = Lerp(size * moonXOffset, size - size * moonXOffset, m_moon.x);
  int ypos = Lerp(size * moonXOffset, maxYPosOfMoon * size, m_moon.y);

  if (colorSet == 3)
  {
    sun.setColor(BackGroundColors::moonColor);
    r = ((m_moon.size * (maxSizeOfMoon - minSizeOfMoon)) + minSizeOfMoon) * size;
  }
  else
  {
    Color color = {0};
    color.r = 255;
    color.g = std::lerp(200, 50, m_moon.y);
    color.b = std::lerp(50, 0, m_moon.y);
    color.a = 255;

    sun.setColor(color);
  }
  sun.draw(xpos, ypos, r);
}

void BackGround::drawMounten(size_t mountenSegments, int min, int max, Color color)
{
  int x = 0;
  int diff = size / (mountenSegments - 1);

  float p = mountenBuff.next();

  int y = Lerp(min, max, p);
  rlSetTexture(1);
  rlBegin(RL_TRIANGLES);
  rlNormal3f(0.0f, 0.0f, 1.0f);
  rlColor4ub(color.r, color.g, color.b, color.a);

  for (size_t i = 1; i <= mountenSegments; i++)
  {
    // topLeft
    rlVertex2f(x, y);
    // bottomLeft
    rlVertex2f(x, size);

    p = mountenBuff.next();
    y = Lerp(min, max, p);
    x += diff;
    // topRight
    rlVertex2f(x, y);

    // bottomRight
    rlVertex2f(x, size);
    // topRight
    rlVertex2f(x, y);

    rlVertex2f(x - diff, size);
  }
  rlEnd();
  rlSetTexture(0);
}